/*********************************************************************************
  *Copyright(C),2015-2022, TUYA www.tuya.comm
  *FileName:    tuya_ipc_album_demo
**********************************************************************************/
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include "tuya_ipc_demo_default_cfg.h"
#include "tuya_ipc_album_demo.h"
#include "tuya_ipc_log_demo.h"
#include "tuya_ipc_p2p.h"
#include "uni_time.h"
#ifdef SUPPORT_ALBUM
#include "tuya_ipc_album.h"

/* Callback functions for transporting events */
int tuya_ipc_stor_album_cb(int event, void* args)
{
    PR_DEBUG("p2p rev event cb=[%d] ", event);
    switch (event)
    {
        /* 内容结构：
        C2C_ALBUM_INDEX_HEAD
        C2C_ALBUM_INDEX_ITEM
        C2C_ALBUM_INDEX_ITEM
        C2C_ALBUM_INDEX_ITEM
        */
        case MEDIA_STREAM_ALBUM_QUERY: /* query album */
        {
            C2C_QUERY_ALBUM_REQ* pSrcType = (C2C_QUERY_ALBUM_REQ*)args;

            // NOTICE: pIndexFile malloc/free in SDK
            int ret = tuya_ipc_album_query_by_name(pSrcType->albumName, pSrcType->channel, &pSrcType->fileLen, (C2C_ALBUM_INDEX_HEAD**)&(pSrcType->pIndexFile));
            if (0 != ret) {
                PR_ERR("err %d", ret); 
            }

            if (pSrcType->pIndexFile) {
                C2C_ALBUM_INDEX_HEAD* ptmp = (C2C_ALBUM_INDEX_HEAD*)pSrcType->pIndexFile;
                PR_DEBUG("get album items %d", ptmp->itemCount);
            }
            break;
        }
        case MEDIA_STREAM_ALBUM_DOWNLOAD_START: /* start download album */
        {
            C2C_CMD_IO_CTRL_ALBUM_DOWNLOAD_START* pSrcType = (C2C_CMD_IO_CTRL_ALBUM_DOWNLOAD_START*)args;
            PR_INFO("start download\n");
            SS_DOWNLOAD_STATUS_E status = 0;
            SS_ALBUM_DOWNLOAD_START_INFO_T strStarInfo = { 0 };
            strStarInfo.session_id = pSrcType->channel;
            memcpy(strStarInfo.album_name, pSrcType->albumName, strlen(pSrcType->albumName));
            strStarInfo.file_count = pSrcType->fileTotalCnt;
            strStarInfo.thumbnail = pSrcType->thumbnail;
            strStarInfo.p_file_info_arr = (SS_FILE_PATH_T*)pSrcType->pFileInfoArr;
            int ret = tuya_ipc_album_set_download_status(SS_DL_START, &strStarInfo);
            if (0 != ret) {
                PR_ERR("err %d", ret);
            }
            break;
        }
        case MEDIA_STREAM_ALBUM_DOWNLOAD_CANCEL: // cancel album
        {
            // thread do nothing...
            C2C_ALBUM_DOWNLOAD_CANCEL* pSrcType = (C2C_ALBUM_DOWNLOAD_CANCEL*)args;
            SS_ALBUM_DOWNLOAD_START_INFO_T strStarInfo = { 0 };
            strStarInfo.session_id = pSrcType->channel;
            memcpy(strStarInfo.album_name, pSrcType->albumName, strlen(pSrcType->albumName));
            int ret = tuya_ipc_album_set_download_status(SS_DL_CANCLE, &strStarInfo);
            if (0 != ret) {
                PR_ERR("err %d", ret);
            }
            PR_ERR("ok %d", ret);

            break;
        }
        case MEDIA_STREAM_ALBUM_DELETE: //delete
        {
            C2C_CMD_IO_CTRL_ALBUM_DELETE* pSrcType = (C2C_CMD_IO_CTRL_ALBUM_DELETE*)args;
            int ret = tuya_ipc_album_delete_by_file_info(pSrcType->channel, pSrcType->albumName, pSrcType->fileNum, (SS_FILE_PATH_T*)pSrcType->pFileInfoArr);
            if (0 != ret) {
                PR_ERR("err %d", ret);
                tuya_ipc_delete_video_finish_v2(pSrcType->channel, TUYA_DOWNLOAD_ALBUM, 0);
            }
            break;
        }
        default:
            return -1;
            break;
    }
    return 0;
}

#ifdef SUPPORT_ALBUM_EMERGECY

/* album info */
extern CHAR_T s_raw_path[128];
#define ABS_PATH_LEN 128
#define CMD_LEN 1024
static char s_test_pic_thumbNail[ABS_PATH_LEN] = "thumbnail/pic_thumbnail.jpg";
static char s_test_mp4_thumbNail[ABS_PATH_LEN] = "thumbnail/mp4_thumbnail.jpg";
static char s_raw_emergency_path[ABS_PATH_LEN] = { 0 };
static char s_raw_emergency_thumbnail[ABS_PATH_LEN] = { 0 };
static char s_emergency_path[ABS_PATH_LEN] = { 0 };
static char s_emergency_thunbnail[ABS_PATH_LEN] = { 0 };

void album_jpg_file_put(char *jpgFile, char *src_path, char *dst_path, char *album_name)
{
    CHAR_T cmd[CMD_LEN];
    int curtime = uni_time_get_posix();
    
    //init target file path
    char targetFile[ABS_PATH_LEN] = { 0 };
    memset(targetFile, 0, sizeof(targetFile));
    sprintf(targetFile, "%d_ch1.jpg", curtime);
    
    //cp pic and thumbnail to emergency album
    memset(cmd, 0, sizeof(cmd));
    sprintf(cmd, "cp %s/%s %s/%s", src_path, jpgFile, dst_path, targetFile);//TODO put jpg into the path
    system(cmd);
    memset(cmd, 0, sizeof(cmd));
    sprintf(cmd, "cp %s/%s %s/%s", src_path, s_test_pic_thumbNail, s_emergency_thunbnail, targetFile);//TODO put thumbnail into the path
    system(cmd);

    // write file info
    ALBUM_FILE_INFO_T strInfo;
    memset(&strInfo, 0, sizeof(ALBUM_FILE_INFO_T));
    strInfo.channel = 0;
    strInfo.type = SS_DATA_TYPE_PIC;
    memcpy(&strInfo.file_name, targetFile, strlen(targetFile));
    strInfo.create_time = curtime;
    strInfo.duration = 0;
    tuya_ipc_album_write_file_info(album_name, &strInfo);

    return;
}

void album_mp4_file_put(char *mp4File)
{
    CHAR_T cmd[CMD_LEN];
    int curtime = uni_time_get_posix();

    //init target file path
    char targetFile[ABS_PATH_LEN] = { 0 };
    memset(targetFile, 0, sizeof(targetFile));
    sprintf(targetFile, "%d_ch2.mp4", curtime);
    char mp4Pic[ABS_PATH_LEN] = { 0 };
    memset(mp4Pic, 0, sizeof(mp4Pic));
    sprintf(mp4Pic, "%d_ch2.jpg", curtime);
    // cp video and thumbnail to emergency album
    memset(cmd, 0, sizeof(cmd));
    sprintf(cmd, "cp %s/%s %s/%s", s_raw_emergency_path, mp4File, s_emergency_path, targetFile);//TODO put mp4 into the path
    PR_INFO("cmd:%s\n",cmd);
    system(cmd);

    memset(cmd, 0, sizeof(cmd));
    sprintf(cmd, "cp %s/%s %s/%s", s_raw_emergency_path, s_test_mp4_thumbNail, s_emergency_thunbnail, mp4Pic);//TODO put thumbnail into the path
     PR_INFO("cmd:%s\n",cmd);
     system(cmd);
    // write file info
    ALBUM_FILE_INFO_T strInfo;
    memset(&strInfo, 0, sizeof(ALBUM_FILE_INFO_T));
    strInfo.channel = 1;
    strInfo.type = SS_DATA_TYPE_VIDOE;
    memcpy(&strInfo.file_name, targetFile, strlen(targetFile));
    strInfo.create_time = curtime;
   // char* p = mp4File + strlen("1234567890_");
    strInfo.duration = 50;// todo check atoi(p);
    tuya_ipc_album_write_file_info(TUYA_IPC_ALBUM_EMERAGE_FILE, &strInfo);

    return;
}

/*
  本地相册文件名要求；
  ipc_emergency_record/
        A.mp4
        B.jpg
        thumbnail/
              A.jpg
              B.jpg
  缩略图文件名和源文件文件名必须相同(除后缀外);
  源文件的文件名不可重名;

  本地相册列表交互ver1:
  1.user获取某相册路径、某相册缩略图路径，
  2.user在相册路径写入源数据（mp4/图片）
  3.user在缩略图路径写入对应缩略图
  4.user通过tuya_ipc_album_write_file_info，更新文件信息
*/
 static int emerge_album_path_inited = 0;
/*
  存放紧急相册文件(jpg/mp4文件)
*/

void album_file_init()
{
    if (0 == emerge_album_path_inited) {
        emerge_album_path_inited = 1;
        tuya_ipc_album_get_path(TUYA_IPC_ALBUM_EMERAGE_FILE, s_emergency_path, s_emergency_thunbnail);
        PR_INFO("get album file path %s \nthumbNail %s\n", s_emergency_path, s_emergency_thunbnail);

        sprintf(s_raw_emergency_path, "%s/resource/ipc_emergency_record", s_raw_path);
        sprintf(s_raw_emergency_thumbnail, "%s/resource/ipc_emergency_record/thumbnail", s_raw_path);
    }

}
void  album_file_put(char *album_name)
{
    //album_file_init();
    CHAR_T cmd[ABS_PATH_LEN];
    char src_path[ABS_PATH_LEN];
    char dst_path[ABS_PATH_LEN];
    char dst_thum_path[ABS_PATH_LEN];

    char* testSrcFile;
    DIR* p_root_dir = NULL;
    struct dirent* p_child_dir = NULL;

    sprintf(src_path, "%s/resource/%s", s_raw_path, album_name);
    tuya_ipc_album_get_path(album_name, dst_path, dst_thum_path);
    /* copy mp4/jpg from src file to album path, and write file info */
    p_root_dir = opendir(src_path);
    if (p_root_dir) {
        while ((p_child_dir = readdir(p_root_dir)) != NULL) {
            if (strcmp(p_child_dir->d_name, ".") == 0
                || strcmp(p_child_dir->d_name, "..") == 0) {
                continue;
            }
            sleep(2);//控制创建速度
            int curtime = uni_time_get_posix();
            testSrcFile = p_child_dir->d_name;
            if (strstr(testSrcFile, "jpg")) {
                album_jpg_file_put(testSrcFile,src_path,dst_path,album_name);
            } else if (strstr(testSrcFile, "mp4")) {
                album_mp4_file_put(testSrcFile);
            }
        }
        closedir(p_root_dir);
    }
}

void album_file_put_v2()
{
    album_file_init();

    int startTime = 1624718683;
    int endTime = 1624718683 + 9;
    int ret;
    // conver record to mp4
    SS_FILE_TIME_TS_T file_info;
    file_info.start_timestamp = startTime; // todo user get from query
    file_info.end_timestamp = endTime;
    char mp4_path_prefix[64] = { 0 };
    char targetFile[48] = { 0 };
    memset(mp4_path_prefix, 0, sizeof(mp4_path_prefix));
    // snprintf(mp4_path_prefix, 64, "%s/%dCh0",s_emergency_path, startTime );// 绝对路径，后缀名由sdk补充完整
    //  snprintf(mp4_path_prefix, 64, "%s/%dCh0",s_emergency_path, startTime );// 绝对路径，后缀名由sdk补充完整
    // 1624761220_ch1_0000.mp4 // todo fileName able to get
    ret = tuya_ipc_ss_save_album_mp4(0,&file_info, s_emergency_path, targetFile, sizeof(targetFile));
    if (0 != ret) {
        PR_INFO("err");
        return;
    }

    //   snprintf(targetFile, 48, "%dCh0_0000.mp4",startTime );
    // write file info
    ALBUM_FILE_INFO_T strInfo;
    memset(&strInfo, 0, sizeof(ALBUM_FILE_INFO_T));
    strInfo.channel = 1;
    strInfo.type = SS_DATA_TYPE_VIDOE;
    memcpy(&strInfo.file_name, targetFile, strlen(targetFile));
    strInfo.create_time = startTime;
    // char* p = mp4File + strlen("1234567890_");
    strInfo.duration = endTime - startTime; // todo check atoi(p);
    tuya_ipc_album_write_file_info(TUYA_IPC_ALBUM_EMERAGE_FILE, &strInfo);
}
#endif

#ifdef SUPPORT_ALBUM_DEFAULT
#define PanoramaFileName "Panorama.jpg"
void  album_file_put_Panorama()
{
    // write file info
    ALBUM_FILE_INFO strInfo;
    memset(&strInfo, 0, sizeof(ALBUM_FILE_INFO));
    strInfo.channel = 1;
    strInfo.type = SS_DATA_TYPE_PANORAMA;
    memcpy(&strInfo.fileName, PanoramaFileName, strlen(PanoramaFileName));
    strInfo.createTime = uni_time_get_posix();;
    strInfo.duration = 0;
    tuya_ipc_album_write_file_info(TUYA_IPC_ALBUM_DEFAULT, &strInfo);

}
#endif

#endif